
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  BSD_exports.pm - psgconf class for generating /etc/exports under Linux
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::BSD_exports;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);

###############################################################################
###  generate() method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;
    my ( $path, $opt, @options, %hosts, $host, $exp );

    $exp = $self->{exports};

    foreach $path ( sort keys %$exp )
    {
        @options = ();
        undef %hosts;

        foreach $opt ( keys %{ $exp->{$path} } )
        {
            if ( !defined( $exp->{$path}->{$opt} ) )
            {
                ### key with no value
                push( @options, $opt );
            } elsif ( !ref( $exp->{$path}->{$opt} ) )
            {
                ### key with scalar value
                push( @options, "$opt=$exp->{$path}->{$opt}" );
            } elsif ( ref( $exp->{$path}->{$opt} ) eq 'HASH' )
            {
                ### key with hash value
                map {
                    $hosts{$_} = []
                      if ( !exists( $hosts{$_} ) );
                    push(
                        @{ $hosts{$_} },
                        (
                            $opt eq 'root'
                            ? 'no_root_squash'
                            : $opt
                        )
                    );
                } keys %{ $exp->{$path}->{$opt} };
            }
        }

        ### We use unshift here instead of push because we want the 'ro'
        ### at the beginning of the list.
        if ( scalar %hosts )
        {
            foreach $host ( keys %hosts )
            {
                unshift( @{ $hosts{$host} }, 'ro' )
                  if ( !grep m/^(ro|rw)$/, @{ $hosts{$host} } );
            }
        } elsif ( scalar @options )
        {
            unshift( @options, 'ro' )
              if ( !grep m/^(ro|rw)$/, @options );
        }

        print $fh $path;

        ### If we have hosts in the hash, then print them out.
        if ( scalar %hosts )
        {
            foreach $host ( sort keys %hosts )
            {
                print $fh ' ' 
                  . $host . '('
                  . join( ',', @{ $hosts{$host} }, @options ) . ')';
            }

            ### If we do not, then we still need to print out the @options
        } elsif ( scalar @options )
        {
            print $fh ' ' . '(' . join( ',', @options ) . ')';
        }

        print $fh "\n";
    }

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::BSD_exports - action to generate /etc/exports

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::BSD_exports;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::BSD_exports->new(
			'name'		=> '/etc/exports',
			'exports'	=> {
				"/path/to/export" => {
					ro	=> { host1, host2, ... },
					rw	=> { host3, host4, ... },
					root	=> { host1, host2, ... },
					anonuid	=> 65534,
					insecure => undef,
					...
				},
				...
			},
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::BSD_exports> module provides a
B<PSGConf> action class for generating the F</etc/exports> file.

The B<PSGConf::Action::GenerateFile::BSD_exports> class is derived from
the B<PSGConf::Action::GenerateFile> class, but it defines/overrides
the following methods:

=over 4

=item generate()

Generates the F</etc/exports> file based on the value of the
I<exports> attribute.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::BSD_exports> class supports the
following attributes:

=over 4

=item I<exports>

A reference to a hash that maps absolute paths to a hash containing the
options for that path to be exported with.

In the sub-hash containing the export options, the value may be a scalar,
in which case it is encoded as the value of the option, or a reference
to a hash, in which case the keys of this sub-sub-hash will be considered
multiple values for the specified option.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut


