
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  PSGConf::Action::GenerateFile::etc_cshrc - action to generate /etc/.cshrc
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::etc_cshrc;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);

###############################################################################
###  generate method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;

    print $fh <<'EOF';

###
### set environment variables
###

EOF
    print $fh 'setenv PATH ' . join( ':', @{ $self->{path} } ) . "\n\n"
      if ( exists( $self->{path} ) );
    print $fh 'setenv MANPATH ' . join( ':', @{ $self->{manpath} } ) . "\n\n"
      if ( exists( $self->{manpath} ) );
    map { print $fh "setenv $_\t\"$self->{env_vars}->{$_}\"\n"; }
      sort keys %{ $self->{env_vars} };

    if ( defined( $self->{shell_aliases} ) )
    {
        print $fh <<'EOF';

###
### set shell aliases
###

EOF
        my ($cmd);
        map {
            ### translate the "<ARGS>" to "\!*"
            $cmd = $self->{shell_aliases}->{$_};
            $cmd =~ s/\<ARGS\>/\\!*/g;

            print $fh "alias $_\t\"$cmd\"\n";
        } ( sort keys %{ $self->{shell_aliases} } );
    }

    if ( defined( $self->{commands} ) )
    {
        print $fh <<'EOF';


###
### other commands
###

EOF
        print $fh "\n$self->{commands}\n";
    }

    print $fh "\n\n";

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::etc_cshrc - generate /etc/.cshrc file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::etc_cshrc;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::etc_cshrc->new(
			'name'		=> '/etc/.cshrc',
			'description'	=> 'csh and tcsh startup file',
			'path'		=> [ ... ],
			'manpath'	=> [ ... ],
			'env_vars'	=> { ... },
			'shell_aliases' => { ... },
			'commands'	=> '...',
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::etc_cshrc> module provides a
B<PSGConf> action class for generating the F</etc/.cshrc> file.

The B<PSGConf::Action::GenerateFile::etc_cshrc> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Creates the F</etc/.cshrc> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::etc_cshrc> class supports the following
attributes:

=over 4

=item I<path>

A list containing directories to be added to the C<PATH> environment
variable.

=item I<manpath>

A list containing directories to be added to the C<MANPATH> environment
variable.

=item I<env_vars>

A hash mapping variables to their values.

=item I<commands>

A string containing commands to be added to the F</etc/.cshrc> file.

=item I<shell_aliases>

A hash containing shell aliases to define.  In the value of aliases,
the string C<E<lt>ARGSE<gt>> will be replaced with C<\!*>.

=back

=head1 SEE ALSO

C<CE<lt>csh(1)E<gt>>

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut

