
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  etc_group.pm - psgconf action to generate /etc/group
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::etc_group;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);

###############################################################################
###  sort function for group_info
###############################################################################

sub _gid_sort
{
    ### first entry should always be root
    return -1
      if ( $a->{name} =~ m/^(root|system)$/ );
    return 1
      if ( $b->{name} =~ m/^(root|system)$/ );

    ### sort identical gids by name
    return ( $a->{name} cmp $b->{name} )
      if ( $a->{gid} == $b->{gid} );

    ### otherwise, sort by gid
    return ( $a->{gid} <=> $b->{gid} );
}

###############################################################################
###  generate() method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;
    my ($entry);

    foreach $entry ( sort _gid_sort values %{ $self->{group_info} } )
    {
        if ( defined $entry->{name} )
        {
            print $fh $entry->{name} . ':';
            print $fh $entry->{passwd} . ':';
            print $fh $entry->{gid} . ':';
            print $fh join( ',', @{ $entry->{members} } )
              if ( exists $entry->{members} );
            print $fh "\n";
        } else
        {
            warn "\tWARNING!!! group has no name, gid => ("
              . $entry->{gid} . ")\n";
        }
    }

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::etc_group - generate /etc/group file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::etc_group;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::etc_group->new(
			'name'		=> '/etc/group',
			'group_info'	=> { ... },
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::etc_group> module provides a B<PSGConf>
action class for generating the F</etc/group> file.

The B<PSGConf::Action::GenerateFile::etc_group> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Creates the F</etc/group> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::etc_group> class supports the following
attributes:

=over 4

=item I<group_info>

A hash mapping group names to a hash containing the fields for that group.
The hash for each group should contain the fields I<name>, which is set
to the name of the group; I<passwd>, which is set to the encrypted
password for the group (usually left empty); I<gid>, which is set to the
numeric gid of the group; and I<members>, which is set to a reference to
a list of group members.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut


