
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  PSGConf::Action::GenerateFile::logrotate - psgconf Action for generating /etc/logrotate.conf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Action::GenerateFile::logrotate;

use strict;

use PSGConf::Action::GenerateFile;
use PSGConf::Util;

our @ISA = qw(PSGConf::Action::GenerateFile);

sub _find_field
{
    my ( $self, $psgconf, $field, $file ) = @_;
    my ( $file2, $res );

    $file2 = &PSGConf::Util::_expand_tokens( $psgconf, $file );

    $res = (
        exists $self->{options}->{$file2}->{$field}
        ? $self->{options}->{$file2}->{$field}
        : (
            exists $self->{options}->{$file}->{$field}
            ? $self->{options}->{$file}->{$field}
            : 'N/A'
        )
    );

    # Translate any special tokens
    $res = &PSGConf::Util::_expand_tokens( $psgconf, $res )
      if ( $res =~ m/%{.*}/ );

    return $res;
}

###############################################################################
###  generate() method
###############################################################################

sub generate
{
    my ( $self, $fh, $psgconf ) = @_;
    my ( $file, $f, $switch, $opt );

    print $fh "create\n";
    print $fh 'rotate ' . $self->{count} . "\n"
      if ( $self->{count} gt 0 );
    print $fh "include /etc/logrotate.d\n";
    print $fh "\n";

    foreach $file ( sort @{ $self->{files} } )
    {

        $f = $file;
        $f = &PSGConf::Util::_expand_tokens( $psgconf, $f )
          if ( $file =~ m/%{.*}/ );

        print $fh "$f {\n";

        # create ...
        my $owner = _find_field( $self, $psgconf, 'owner', $file );
        my $group = _find_field( $self, $psgconf, 'group', $file );
        my $mode  = _find_field( $self, $psgconf, 'mode',  $file );
        print $fh "\tcreate $mode $owner $group\n"
          if ( $owner ne 'N/A' && $group ne 'N/A' && $mode ne 'N/A' );

        # All switches that do not take arguments
        foreach $switch ( 'compress', 'copy', 'missingok', 'nocompress' )
        {
            $opt = _find_field( $self, $psgconf, $switch, $file );
            print $fh "\t$switch\n"
              if ( $opt ne 'N/A' );
        }

        # Now switches that do...
        foreach $switch ('size')
        {
            $opt = _find_field( $self, $psgconf, $switch, $file );
            print $fh "\t$switch $opt\n"
              if ( $opt ne 'N/A' );
        }

        # Now a switch that takes its value as a boolean...
        foreach $switch ( 'period', 'template' )
        {
            $opt = _find_field( $self, $psgconf, $switch, $file );
            print $fh "\t$opt\n"
              if ( $opt ne 'N/A' );
        }

        # count => rotate
        $opt = _find_field( $self, $psgconf, 'count', $file );
        print $fh "\trotate $opt\n"
          if ( $opt ne 'N/A' );

        # postrotate ...
        $opt = _find_field( $self, $psgconf, 'post_command', $file );
        print $fh "\tpostrotate\n\t\t$opt\n\tendscript\n\tsharedscripts\n"
          if ( $opt ne 'N/A' );

        print $fh "}\n\n";
    }

    return 1;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::logrotate - generate /etc/logrotate.conf file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::logrotate;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::logrotate->new(
			'name'	=> '/etc/logrotate.conf',
			'uid'	=> 0,
			'options'	=> { ... },
			'count' => 14,
			'files'	=> [ ... ],
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::logrotate> module provides a B<PSGConf>
action class for generating the F</etc/logrotate.conf> file.

The B<PSGConf::Action::GenerateFile::logrotate> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Creates the F</etc/logrotate.conf> file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::logrotate> class supports the following
attributes:

=over 4

=item I<name>

The name of the file to generate, usually F</etc/logrotate.conf>.

=item I<uid>

What owner the F<logrotate.conf> file should be.

=item I<options>

Any extra options that are required to be put into F<logrotate.conf>.

=over 4

=item I<post_command>

If set, the logfile rotation tools should execute this command after the
file is rotated.  Maps to the C<postrotate> ... C<endscript> C<sharedscripts>
commands in the config file.

=item I<count>

An integer for the number of files to keep.  Maps to the C<rotate> command
in the config file.

=item I<copy>

Tells F<logrotate> to copy the file istead of moving it.

=item I<group>

What the group members should be for the log file.  Maps to the C<create>
command in the config file, if C<mode> and C<owner> are also set.

=item I<mode>

What the mode should be for the log file.  Maps to the C<create>
command in the config file, if C<group> and C<owner> are also set.

=item I<missingok>

Tells F<logrotate> that if the file does not exist, do not complain about it.

=item I<owner>

Who the owner of the file should be.

=item I<period>

How often the log files are rotated.

=item I<size>

Rotate the log file when it reaches this size.

=item I<template>

What special extension that should be assigned to the file.  The only
supported value is I<dateext>.

=item I<compress>

If this key exists, it will cause the log files to be compressed after
they are rotated. 

=item I<nocompress>

If this key exists, it will cause the log files not to be compressed after
they are rotated. 

=back

=item I<count>

The default number of files we keep.

=item I<files>

The list of files we rotate.

=back

=head1 SEE ALSO

C<CE<lt>logrotate(8)E<gt>>

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut
