
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  RPC.pm - RPC configuration module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Control::RPC;

use PSGConf::Action::svcs::setprop;
use PSGConf::Action::GenerateFile::etc_rpc;
use PSGConf::Data::Boolean;
use PSGConf::Data::String;
use PSGConf::Data::Table;
use PSGConf::Data::List;
use PSGConf::Control::Packages qw(_add_pkgs);
use PSGConf::Util qw(get_addrs);

use strict;

###############################################################################
###  decide() method
###############################################################################

sub decide
{
    my ( $self, $psgconf ) = @_;

    return
      if ( $psgconf->data_obj('rpc_enable')->equals('false') );

    $psgconf->register_actions(
        PSGConf::Action::GenerateFile::etc_rpc->new(
            'name'        => '/etc/rpc',
            'description' => 'RPC configuration file',
            entries       => $psgconf->data_obj('rpc')->get()
        )
    );

    if ( $psgconf->data_obj('platform')->match('solaris10') )
    {
        $psgconf->register_actions(
            PSGConf::Action::svcs::setprop->new(
                name     => 'enable remote rpcbind connections',
                FMRI     => 'svc:/network/rpc/bind',
                property => 'config/local_only',
                value    => 'false'
            ),
            PSGConf::Action::svcs::setprop->new(
                name     => 'enable rpcbind verbose logging',
                FMRI     => 'svc:/network/rpc/bind',
                property => 'config/verbose_logging',
                value    => 'true'
            ),
            PSGConf::Action::svcs::setprop->new(
                name => 'enable TCP Wrappers for '
                  . $psgconf->data_obj('rpcbind_cmd')->get(),
                FMRI     => 'svc:/network/rpc/bind',
                property => 'config/enable_tcpwrappers',
                value    => 'true'
            )
        );
    }
}

###############################################################################
###  policy methods
###############################################################################

### add TCP wrappers entry
sub _policy_add_tcpwrapper_entry
{
    my ( $self, $psgconf ) = @_;
    my ( $hosts, $cmd );

    return
      if ( $psgconf->data_obj('rpc_enable')->equals('false') );

    $cmd = $psgconf->data_obj('rpcbind_cmd')->get();

    ### Need to translate to IP addresses so that
    ### tcp_wrappers works correctly.
    map {
        $hosts .= ' ' if ( defined $hosts );
        $hosts .= join( ' ', &get_addrs( $psgconf, $_ ) );
    } @{ $psgconf->data_obj('rpcbind_clients')->get() };

    if (
        !$psgconf->data_obj('tcp_wrappers')->find_row( { 0 => qr/\b$cmd\b/ } ) )
    {
        $psgconf->data_obj('tcp_wrappers')
          ->insert_row( { 1 => 'all' }, [ $cmd, $hosts, 'allow' ] );
    } else
    {
        $psgconf->data_obj('tcp_wrappers')
          ->append_to_row_cells( { 0 => qr/\b$cmd\b/ }, { 1 => $hosts } );
    }
}

sub _enable_rc_scripts
{
    my ( $self, $psgconf ) = @_;
    my ($cmd);

    $cmd = $psgconf->data_obj('rpcbind_cmd')->get();

    $psgconf->data_obj('rc_scripts')
      ->insert( { $cmd => { 'state' => 'enable' } } )
      if ( $psgconf->data_obj('rpc_enable')->equals('true') );
}

###############################################################################
###  constructor
###############################################################################

sub new
{
    my ( $class, $psgconf ) = @_;
    my ($self);

    $self = {};
    bless( $self, $class );

    ### So that _add_pkgs knows which directives to look at
    $self->{name}     = 'rpc';
    $self->{enable}   = $self->{name} . '_enable';
    $self->{packages} = $self->{name} . '_packages';

    $psgconf->register_data(
        rpc_enable      => PSGConf::Data::Boolean->new( value => 'false' ),
        rpcbind_cmd     => PSGConf::Data::String->new( value  => 'rpcbind' ),
        rpcbind_clients => PSGConf::Data::List->new(),
        rpc             => PSGConf::Data::Table->new(),
        rpc_packages    => PSGConf::Data::List->new()
    );

    $psgconf->register_policy(
        $self,
        rpc_add_packages         => '_add_pkgs',
        rpc_add_tcpwrapper_entry => '_policy_add_tcpwrapper_entry',
        rpc_enable_rc_scripts    => '_enable_rc_scripts'
    );

    return $self;
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::RPC - psgconf control class for RPC configuration

=head1 SYNOPSIS

In F<psgconf_modules>:

  Control PSGConf::Control::RPC

=head1 DESCRIPTION

The B<PSGConf::Control::RPC> module provides a B<psgconf> control object
for configuring RPC services.  It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<rpc_enable>

A B<PSGConf::Data::Boolean> object enabling the RPC module.

=item I<rpcbind_cmd>

A B<PSGConf::Data::String> object containing the name of the rpcbind
or portmapper command.  Defaults to C<rpcbind>.

=item I<rpcbind_clients>

A B<PSGConf::Data::List> object containing the list of hosts to allow
to talk to rpcbind/portmap.

=item I<rpc_packages>

A B<PSGConf::Data::List> object containing the packages to install.

=item I<rpc>

A B<PSGConf::Data::Table> object containing the list of rpcs to put
into the F</etc/rpc> file.

=back

The constructor also registers the following policy method:

=over 4

=item I<rpc_add_packages>

Adds the packges listed in the C<rpc_packages> directive to the
C<pkg_install_list> list to be installed by the C<Packages>
Control module.

=back

=item decide()

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::svcs::setprop>

L<PSGConf::Action::GenerateFile::etc_rpc>

L<PSGConf::Control::Packages>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::List>

L<PSGConf::Data::String>

L<PSGConf::Data::Table>

L<PSGConf::Util>

L<psgconf-intro>

=cut

