
###
###  Copyright 2000-2008 University of Illinois Board of Trustees
###  All rights reserved.
###
###  swatch.pm - swatch module for psgconf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###

package PSGConf::Control::swatch;

use File::Basename;
use strict;

use PSGConf::Action::GenerateFile::Literal;
use PSGConf::Action::MkDir;
use PSGConf::Data::Boolean;
use PSGConf::Data::Hash;
use PSGConf::Data::List;
use PSGConf::Data::String;
use PSGConf::Control::Packages qw(_add_pkgs);
use PSGConf::Util;

###############################################################################
###  Utility function to be called from other Control modules
###############################################################################

require Exporter;
our @ISA         = qw (Exporter);
our %EXPORT_TAGS = ( 'all' => [qw( _add_swatch_rule _remove_swatch_rules)] );
our @EXPORT_OK   = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT      = qw();

sub _add_swatch_rule
{
    my ( $self, $psgconf ) = @_;

    if (   !exists( $self->{swatch_key} )
        || $self->{swatch_key} eq ''
        || !exists( $self->{swatch_rule} )
        || $self->{swatch_rule} eq '' )
    {
        warn "\t!!!Control::swatch->_add_swatch_rule(): no attribute(s)\n";
        return;
    }

    $psgconf->data_obj('swatch_config')
      ->insert( { $self->{swatch_key} => [ $self->{swatch_rule} ] } );
}

sub _remove_swatch_rules
{
    my ( $self, $psgconf ) = @_;

    if ( !exists( $self->{swatch_key} ) || $self->{swatch_key} eq '' )
    {
        warn "\t!!!Control::swatch->_remove_swatch_rules(): no attribute(s)\n";
        return;
    }
    $psgconf->data_obj('swatch_config')->delete( $self->{swatch_key} );
}

###############################################################################
###  policy methods
###############################################################################

###
### WARNING: Needs to be called after syslog_default_dir policy is called,
### because syslog_dir may get prepended there.
###
sub _policy_add_crontab
{
    my ( $self, $psgconf ) = @_;
    my ( $cmd, $file );

    return
      if ( !-f $psgconf->data_obj('swatch_cmd')->get()
        || $psgconf->data_obj('swatch_enable')->equals('false') );

    foreach $file ( keys %{ $psgconf->data_obj('swatch_config')->get() } )
    {
        $cmd =
            $psgconf->data_obj('swatch_cmd')->get() . ' -c '
          . $psgconf->data_obj('swatch_dir')->get() . '/'
          . basename($file) . ' -f '
          . $file
          . ' > /dev/null 2>&1';

        $psgconf->data_obj('crontabs')->insert(
            {
                'root' => [
                    {
                        hour    => $psgconf->data_obj('swatch_hour')->get(),
                        minute  => $psgconf->data_obj('swatch_minute')->get(),
                        command => $cmd
                    }
                ]
            }
        );
    }
}

### set absolute path to log files
sub _policy_canonify_logfiles
{
    my ( $self, $psgconf ) = @_;
    my ( $file, $new_file );

    return
      if ( $psgconf->data_obj('swatch_enable')->equals('false') );

    foreach $file ( keys %{ $psgconf->data_obj('swatch_config')->get() } )
    {

        # If we have the special token %{DIR}, expand the value of DIR
        if ( $file =~ m|^%{.*}| )
        {
            $new_file = PSGConf::Util::_expand_tokens( $psgconf, $file );
            $psgconf->data_obj('swatch_config')->insert(
                {
                    $new_file =>
                      $psgconf->data_obj('swatch_config')->find($file)
                }
            );
            $psgconf->data_obj('swatch_config')->delete($file);
        }
    }
}

###
### WARNING: This is a fix because Solaris systems complain about a host
### being our address, when you look into the MAC address it reports, that
### is the same host's MAC address.
###
sub _policy_add_swatch_rule
{
    my ( $self, $psgconf ) = @_;
    my ( @mac, @ip, $i, @res, $line, $tmp );

    ### If we are not running swatch nor on Solaris, just return
    return
      if (!$psgconf->data_obj('platform')->match('solaris')
        || $psgconf->data_obj('swatch_enable')->equals('false') );

    ### find all the IP/MAC pairs that we are currently using
    $i   = -1;
    @res = `/sbin/ifconfig -au`;
    foreach $line (@res)
    {
        if ( $line =~ /^\s+inet / )
        {
            $i++;
            $tmp = $line;
            $tmp =~ s/^\s*inet //;
            $tmp =~ s/\s.*$//;
            $ip[$i] = &_format_addr( $tmp, 'IP' );
        }

        if ( $line =~ /^\s+ether / )
        {
            my ($tmp) = $line;
            $tmp =~ s/^\s*ether //;
            $mac[$i] = &_format_addr( $tmp, 'MAC' );
        }
    }

    ### Now assemble the swatch configuration for all the IP/MAC pairs
    $line = undef;
    for ( $i = 0 ; $i < scalar(@ip) ; $i++ )
    {
        $line .=
            "ignore / WARNING: IP: Hardware address '"
          . $mac[$i]
          . "' trying to be our address "
          . $ip[$i] . "!/\n"
          if ( defined $ip[$i] && defined $mac[$i] );
    }
    $self->{swatch_key} = "%{syslog_dir}/messages";
    $self->{swatch_rule} =
        "# Ignore warnings reported between my IP and MAC address\n" 
      . $line
      . "	continue";

    $self->_add_swatch_rule($psgconf)
      if ( defined $line );
}

sub _format_addr
{
    my ( $str, $type ) = @_;
    my ( $res, @addr, $sep, $cnt, $t, $digits );

    if ( $type eq 'IP' )
    {
        $sep    = '.';
        $cnt    = 4;
        $digits = 3;
        $t      = 'd';
    } elsif ( $type eq 'MAC' )
    {
        $sep    = ':';
        $cnt    = 6;
        $digits = 2;
        $t      = 'x';
    } else
    {
        return undef;
    }

    @addr = split( /\Q$sep\E/, $str );

    return undef
      if ( $cnt != scalar(@addr) );

    $res = undef;
    for ( my $i = 0 ; $i < scalar(@addr) ; $i++ )
    {
        $res .= $sep if ( defined $res );
        $res .=
          sprintf "%${digits}.${digits}$t",
          ( $type eq 'MAC' )
          ? hex( $addr[$i] )
          : $addr[$i];
    }

    return $res;
}

###############################################################################
###  Constructor
###############################################################################

sub new
{
    my ( $class, $psgconf ) = @_;
    my ($self);

    $self = {};
    bless( $self, $class );

    $self->{name}     = 'swatch';
    $self->{enable}   = $self->{name} . '_enable';
    $self->{packages} = $self->{name} . '_packages';

    $psgconf->register_data(
        swatch_enable => PSGConf::Data::Boolean->new( value   => 'false' ),
        swatch_config => PSGConf::Data::Hash->new( value_type => 'ARRAY' ),
        swatch_dir    => PSGConf::Data::String->new(
            'value_abspath' => 1,
            value           => '/usr/local/etc/swatch'
        ),
        swatch_cmd => PSGConf::Data::String->new(
            'value_abspath' => 1,
            value           => '/usr/local/bin/swatch'
        ),
        swatch_mailto   => PSGConf::Data::String->new( value => 'root' ),
        swatch_hour     => PSGConf::Data::String->new( value => '23' ),
        swatch_minute   => PSGConf::Data::String->new( value => '58' ),
        swatch_packages => PSGConf::Data::List->new()
    );

    $psgconf->register_policy(
        $self,
        swatch_add_packages      => '_add_pkgs',
        swatch_canonify_logfiles => '_policy_canonify_logfiles',
        swatch_add_crontab       => '_policy_add_crontab',
        swatch_add_rule          => '_policy_add_swatch_rule'
    );

    return $self;
}

##############################################################################
###  decide() method
###############################################################################

sub decide
{
    my ( $self, $psgconf ) = @_;

    return
      if ( $psgconf->data_obj('swatch_enable')->equals('false') );

    $psgconf->register_actions(
        PSGConf::Action::MkDir->new(
            name => $psgconf->data_obj('swatch_dir')->get(),
            mode => 0750
        ),
        (
            map {
                PSGConf::Action::GenerateFile::Literal->new(
                    name => $psgconf->data_obj('swatch_dir')->get() . '/'
                      . basename($_),
                    mode        => 0640,
                    description => 'Swatch Configuration File',
                    content     => join( "\n\n",
                        @{ $psgconf->data_obj('swatch_config')->find($_) } )
                      . "\n\nwatchfor /.*/\n\tmail addresses="
                      . $psgconf->data_obj('swatch_mailto')->get()
                      . ",subject=Message from Swatch on "
                      . $psgconf->data_obj('hostname')->get() . ":$_"
                      . ",keep_open=1\n"
                  )
              } keys %{ $psgconf->data_obj('swatch_config')->get() }
        )
    );
}

###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Control::swatch - psgconf control class for swatch syslog reporting

=head1 SYNOPSIS

In F</etc/psgconf_modules>:

  Control PSGConf::Control::swatch

=head1 DESCRIPTION

The B<PSGConf::Control::swatch> module provides a B<psgconf> control object
for configuring swatch syslog reporting.  It supports the following methods:

=over 4

=item new()

The constructor.  Its parameter is a reference to the B<PSGConf>
object.  It registers the following data objects:

=over 4

=item I<swatch_enable>

A B<PSGConf::Data::Boolean> object, indicating whether swatch should
be configured.  The default is no.

=item I<swatch_packages>

A B<PSGConf::Data::List> object containing a list of packages to install.

=item I<swatch_config>

A B<PSGConf::Data::Hash> object containing a key of what file to have swatch
process and the value is an anomyous list of what actions to put into the 
F<etc/swatch/KEY> file.

=item I<swatch_mailto>

A B<PSGConf::Data::String> object containing an email address
to send the results of the C<swatch> command to.

=item I<swatch_dir>

A B<PSGConf::Data::String> object containing the directory to store the
F<swatchrc> files in.  Defaults to F</usr/local/etc/swatch>.

=item I<swatch_cmd>

A B<PSGConf::Data::String> object containing the full command to the
F<swatch> binary.  Defaults to F</usr/local/bin/swatch>.

=item I<swatch_hour>

A B<PSGConf::Data::String> object containing which hour or hours
to run F<swatch>.  Defaults to C<23>.

=item I<swatch_minute>

A B<PSGConf::Data::String> object containing which minute or minutes
to run F<swatch>.  Defaults to C<58>.

=back

The constructor also registers the following policy methods:

=over 4

=item I<swatch_add_packages>

If I<swatch_enable> is true, adds the B<swatch> package and its
requirements to the I<pkg_install_list> Data object (provided by
B<PSGConf::Control::Packages>).

=item I<bass_add_crontab>

If I<swatch_enable> is true, adds an entry for the I<swatch> nightly
to I<root>'s I<crontab> Data object (provided by B<PSGConf::Control::cron>).

=item decide()

Does the following:

If I<swatch_enable> is true, creates the F</usr/local/etc/swatchrc>
config file for the C<swatch> command to use.

=back

=back

=head1 SEE ALSO

L<perl>

L<File::Basename>

L<PSGConf>

L<PSGConf::Action::GenerateFile::Literal>

L<PSGConf::Action::MkDir>

L<PSGConf::Control::cron>

L<PSGConf::Control::Packages>

L<PSGConf::Data::Boolean>

L<PSGConf::Data::Hash>

L<PSGConf::Data::List>

L<PSGConf::Data::String>

L<psgconf-intro>

=cut

